/**
 * this library implement UPNP Device Architecture 1.1
 * Version 1.0 implement step 1 : discovery
 * other steps will be implementer later.
 *
 * Step 1 purpose : When a device is added to the network,
 * the UPnP discovery protocol allows that device to advertise its services
 * to control points on the network. 
 * Similarly, when a control point is added to the network, the UPnP discovery
 * protocol allows that control point to search for devices of interest 
 * on the network.
 * The fundamental exchange in both cases is a discovery message containing a
 * few essential specifics about the device or one of its services, e.g.,
 * its type, identifier, and a pointer to more detailed information. 
 *
 * author dominique Desmet
 * version 1.0
 * date september 2016
 * license : ISC
 * ISC License
 * Copyright (c) dominique desmet
 * Permission to use, copy, modify, and/or distribute this software for any purpose with or without fee
 * is hereby granted, provided that the above copyright notice and this permission notice appear in all copies.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE
 * INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS. 
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES
 * OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT,
 * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE
 * OF THIS SOFTWARE.
 **/
 var SSDPprocessor = require("./lib/discover/upnp_da11_ssdp.js");
 var STall = require("./lib/discover/upnp_da11_st_all.js");
 var STroot = require("./lib/discover/upnp_da11_st_root.js");
 var STdeviceType = require("./lib/discover/upnp_da11_st_device_type.js");
 var STserviceType = require("./lib/discover/upnp_da11_st_service_type.js");
 var STuuid = require("./lib/discover/upnp_da11_st_uuid.js");
 var STvendorDeviceType = require("./lib/discover/upnp_da11_st_vendor_device_type.js");
 var STvendorServiceType = require("./lib/discover/upnp_da11_st_vendor_service_type.js");
 var UPnPdeviceDescription = require("./lib/description/upnp_da11_device_description.js"); 

   
 var SSDP = new SSDPprocessor();
 var STall = new STall();
 var STroot = new STroot();
 
 module.exports = {
   getSSDP                : function(){ return SSDP;},
   getSTall               : function(){return STall.ST;},
   getSTroot              : function(){return STroot.ST;},
   getSTdeviceType        : function(dt){return new STdeviceType(dt).ST;},
   getSTserviceType       : function(st){return new STserviceType(st).ST;},
   getSTuuid              : function(uuid) {return new STuuid(uuid).ST;},
   getSTvendorDeviceType  : function(vd,vdt) {return new STvendorDeviceType(vd,vdt).ST;},
   getSTvendorServiceType : function(vd,vst) {return new STvendorServiceType(vd,vst).ST;},
   getUPnPdeviceDescription:function(url) {return new UPnPdeviceDescription(url);}
 }
 