const Dgram = require("dgram"),
      EventEmitter = require("events").EventEmitter,
      NotifyMessage=require("./upnp_da11_notify_message"),
      MSOKMessage=require("./upnp_da11_msearchok_message"),
      MsearchMessage=require("./upnp_da11_msearch_message"),
      SSDPError=require("../error/upnp_da11_ssdp_error");

class SSDPprocessor  {
  constructor(){
    
    var listenerSSDP = new EventEmitter();
    
    var parse = function(msg){
      var context = this;
      var parsed = {};
      var headers = msg.toString('ascii').split('\r\n');
      var typeh = headers[0];
      headers.shift();        
      for(var i = 0; i<headers.length; i++) {
        var splitCharIndex = headers[i].indexOf(':');
        if (splitCharIndex > 0) {
          parsed[headers[i].slice(0,splitCharIndex).toUpperCase().trim()] = headers[i].slice(splitCharIndex + 1).trim();
        }
      }
      if (typeh=="NOTIFY * HTTP/1.1") {
        try {
          var notify = new NotifyMessage(parsed);
          listenerSSDP.emit(typeh,notify);
        } catch(error){
          throw error;
        }
      } else if (typeh=="M-SEARCH * HTTP/1.1"){
        try {
          var msearch = new MsearchMessage(parsed);
          listenerSSDP.emit(typeh,msearch);
        } catch(error){
          throw error;        }
      } else if (typeh=="HTTP/1.1 200 OK") {
        try {
          var msearchok = new MSOKMessage(parsed);
          listenerSSDP.emit(typeh,msearchok); 
        }catch(error){
          throw error;
        }       
      }
    }

    /* create udp socket*/
    var udp=Dgram.createSocket("udp4");
    
    /* throw udp errors */
    udp.on("error", (error) => {
        throw(error);
    })
    
    /* process ssdp received messages */
    udp.on('message',  (msg, rinfo) =>{
      listenerSSDP.emit("RAW",msg); 
      parse(msg);
    })
 
    udp.bind(1900, () => {
      udp.addMembership("239.255.255.250");
    })
         
    this.search = function(ST){
      console.log(ST);
      var CRLF = '\r\n';
      var searchTarget=
       "M-SEARCH * HTTP/1.1"+CRLF
      +"HOST: 239.255.255.250:1900"+CRLF
      +"MX: 3"+CRLF
      +"MAN: \"ssdp:discover\""+CRLF
      +"ST: "+ST+CRLF+CRLF;
      var st = new Buffer(searchTarget,'ascii');
      udp.send(searchTarget,0,searchTarget.length,1900,"239.255.255.250");         
    }
    
    this.getListener=function(){return listenerSSDP;}
  }
  
} 

module.exports=SSDPprocessor;